import React, { FC, useCallback, useEffect, useState } from 'react';
import { format } from 'date-fns';

import { ClockTimeIcon } from '@procyon/components/Icons/ClockTime';
import { useModal } from '@procyon/components/Modal/hooks';
import { InputElementType } from '@procyon/types/form';
import { Option } from '@procyon/types/options';

import { TextField } from '../../TextField';
import { TimePickerModal } from './TimePickerModal';

export type TimePickerProps<V = Date> = Omit<
	{
		hour?: {
			max?: number;
			min?: number;
			step?: number;
		};
		minute?: {
			max?: number;
			min?: number;
			step: number;
		};
		scrollToCode?: Option['code'];
	} & InputElementType<V>,
	'onFocus' | 'onClick' | 'onBlur'
>;

export const TimePicker: FC<TimePickerProps> = (props) => {
	const { onChange, label, name, hour, minute, scrollToCode, ...others } = props;

	const { openModal, registerModal } = useModal();

	const [value, setValue] = useState<Date>();

	useEffect(() => {
		setValue(props.value);
	}, [props.value]);

	const handleTimeClick = (value: Date): void => {
		setValue(value);
		onChange?.(value, null);
	};

	const handleOpenModal = (): void => {
		registerModal({
			id: name,
			title: label,
			style: { width: 200 },
			Component: modalComponent
		});
		openModal(name);
	};

	const modalComponent = useCallback(
		(props) => (
			<TimePickerModal
				value={value}
				onChange={handleTimeClick}
				scrollToCode={scrollToCode}
				modalName={name}
				minute={{ step: 5, min: 0, max: 59, ...(minute ?? {}) }}
				hour={{ step: 1, min: 0, max: 23, ...(hour ?? {}) }}
				{...props}
			/>
		),
		[value, scrollToCode, name, minute, hour]
	);

	return (
		<TextField
			{...others}
			name={name}
			label={label}
			value={value ? format(value, 'HH:mm') : undefined}
			leftIcon={<ClockTimeIcon onClick={handleOpenModal} className="field-icon" />}
			placeholder="HH:mm"
			mask="99:99"
		/>
	);
};
